#ifndef _MATCHER_H_
#define _MATCHER_H_

#include <Common/SEGlobal.h>

/**
 * Structure that represents a match result.
 */
class tMatchResult {
public:
	// ID for matched document
	tDocID docID;
	// Number of matched tokens within this document
	tScore matchScore;

	tMatchResult() {}

	tMatchResult(tDocID _docID) : docID(_docID), matchScore(1) {}

	bool operator<(const tMatchResult& other) const { return (docID < other.docID); }
};

inline std::ostream& operator<<(std::ostream& s, const tMatchResult& matchResult) {
	s << "(" << matchResult.docID << "#" << matchResult.matchScore << ")";
	return s;
}

/**
 * Base-class for document matchers.
 *
 * Matchers are responsible for matching the query
 * with the relevant documents.
 */
class Matcher {
protected:
	UINT m_maxToMatch;
public:
	Matcher(UINT maxToMatch) : m_maxToMatch(maxToMatch) {}

	/**
	 * Matches the given queryTokens 
	 * @param queryTokens - Tokens from the query to match
	 * @param result      - Vector for matched documents
	 * @param maxToMatch  - Maximum number of matches (=0 indicates no maximum),
	 *                      Other documents are discarded
	 * @return total number of documents matched
	 */
	virtual int match(const Vector<tTerm>& queryTerms, Set<tMatchResult>& result) = 0;
};

#endif /* _MATCHER_H_ */